@@ -290,11 +290,11 @@ int du_main(int argc UNUSED_PARAM, char **argv)
 	total = 0;
 	do {
 		total += du(*argv);
-		/* otherwise du /dir /dir won't show /dir twice: */
-		reset_ino_dev_hashtable();
 		G.slink_depth = slink_depth_save;
 	} while (*++argv);
 
+	if (ENABLE_FEATURE_CLEAN_UP)
+		reset_ino_dev_hashtable();
 	if (opt & OPT_c_total)
 		print(total, "total");
 
