@@ -5,7 +5,7 @@
  * Licensed under GPLv2, see file LICENSE in this source tree.
  */
 //config:config NL
-//config:	bool "nl (4.3 kb)"
+//config:	bool "nl (4.6 kb)"
 //config:	default y
 //config:	help
 //config:	nl is used to number lines of files.
@@ -58,6 +58,8 @@ int nl_main(int argc UNUSED_PARAM, char **argv)
 		"number-width\0"	Required_argument "w"
 	;
 #endif
+	int exitcode;
+
 	ns.width = 6;
 	ns.start = 1;
 	ns.inc = 1;
@@ -72,9 +74,10 @@ int nl_main(int argc UNUSED_PARAM, char **argv)
 	if (!*argv)
 		*--argv = (char*)"-";
 
+	exitcode = EXIT_SUCCESS;
 	do {
-		print_numbered_lines(&ns, *argv);
+		exitcode |= print_numbered_lines(&ns, *argv);
 	} while (*++argv);
 
-	fflush_stdout_and_exit(EXIT_SUCCESS);
+	fflush_stdout_and_exit(exitcode);
 }
