@@ -9,12 +9,16 @@
  * Rani Assaf <rani@magic.metawire.com> 980929: resolve addresses
  * Kunihiro Ishiguro <kunihiro@zebra.org> 001102: rtnh_ifindex was not initialized
  */
-
 #include "ip_common.h"  /* #include "libbb.h" is inside */
 #include "common_bufsiz.h"
 #include "rt_names.h"
 #include "utils.h"
 
+#include <linux/version.h>
+/* RTA_TABLE is not a define, can't test with ifdef. */
+/* As a proxy, test which kernels toolchain expects: */
+#define HAVE_RTA_TABLE (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19))
+
 #ifndef RTAX_RTTVAR
 #define RTAX_RTTVAR RTAX_HOPS
 #endif
@@ -22,7 +26,10 @@
 
 struct filter_t {
 	int tb;
+	/* Misnomer. Does not mean "flushed something" */
+	/* More like "flush commands were constructed by print_route()" */
 	smallint flushed;
+	/* Flush cmd buf. If !NULL, print_route() constructs flush commands in it */
 	char *flushb;
 	int flushp;
 	int flushe;
@@ -49,7 +56,7 @@ typedef struct filter_t filter_t;
 
 static int flush_update(void)
 {
-	if (rtnl_send(G_filter.rth, G_filter.flushb, G_filter.flushp) < 0) {
+	if (rtnl_send_check(G_filter.rth, G_filter.flushb, G_filter.flushp) < 0) {
 		bb_perror_msg("can't send flush request");
 		return -1;
 	}
@@ -79,12 +86,14 @@ static int FAST_FUNC print_route(const struct sockaddr_nl *who UNUSED_PARAM,
 	if (len < 0)
 		bb_error_msg_and_die("wrong nlmsg len %d", len);
 
-	memset(tb, 0, sizeof(tb));
+	//memset(tb, 0, sizeof(tb)); - parse_rtattr does this
 	parse_rtattr(tb, RTA_MAX, RTM_RTA(r), len);
 
+#if HAVE_RTA_TABLE
 	if (tb[RTA_TABLE])
 		tid = *(uint32_t *)RTA_DATA(tb[RTA_TABLE]);
 	else
+#endif
 		tid = r->rtm_table;
 
 	if (r->rtm_family == AF_INET6)
@@ -460,12 +469,13 @@ IF_FEATURE_IP_RULE(ARG_table,)
 			NEXT_ARG();
 			if (rtnl_rttable_a2n(&tid, *argv))
 				invarg_1_to_2(*argv, keyword_table);
-			if (tid < 256)
-				req.r.rtm_table = tid;
-			else {
+#if HAVE_RTA_TABLE
+			if (tid > 255) {
 				req.r.rtm_table = RT_TABLE_UNSPEC;
 				addattr32(&req.n, sizeof(req), RTA_TABLE, tid);
-			}
+			} else
+#endif
+				req.r.rtm_table = tid;
 #endif
 		} else if (arg == ARG_dev || arg == ARG_oif) {
 			NEXT_ARG();
@@ -1074,7 +1084,7 @@ static int iproute_get(char **argv)
 			bb_error_msg_and_die("wrong len %d", len);
 		}
 
-		memset(tb, 0, sizeof(tb));
+		//memset(tb, 0, sizeof(tb)); - parse_rtattr does this
 		parse_rtattr(tb, RTA_MAX, RTM_RTA(r), len);
 
 		if (tb[RTA_PREFSRC]) {
