@@ -94,19 +94,19 @@ enum {
 	IACBUFSIZE  = 128,
 
 	CHM_TRY = 0,
-	CHM_ON = 1,
+	CHM_ON  = 1,
 	CHM_OFF = 2,
 
 	UF_ECHO = 0x01,
-	UF_SGA = 0x02,
+	UF_SGA  = 0x02,
 
 	TS_NORMAL = 0,
 	TS_COPY = 1,
-	TS_IAC = 2,
-	TS_OPT = 3,
+	TS_IAC  = 2,
+	TS_OPT  = 3,
 	TS_SUB1 = 4,
 	TS_SUB2 = 5,
-	TS_CR = 6,
+	TS_CR   = 6,
 };
 
 typedef unsigned char byte;
@@ -152,8 +152,10 @@ static void subneg(byte c);
 
 static void iac_flush(void)
 {
-	full_write(netfd, G.iacbuf, G.iaclen);
-	G.iaclen = 0;
+	if (G.iaclen != 0) {
+		full_write(netfd, G.iacbuf, G.iaclen);
+		G.iaclen = 0;
+	}
 }
 
 static void doexit(int ev) NORETURN;
@@ -244,25 +246,34 @@ static void handle_net_output(int len)
 
 static void handle_net_input(int len)
 {
+	byte c;
 	int i;
 	int cstart = 0;
 
-	for (i = 0; i < len; i++) {
-		byte c = G.buf[i];
-
-		if (G.telstate == TS_NORMAL) { /* most typical state */
-			if (c == IAC) {
-				cstart = i;
-				G.telstate = TS_IAC;
-			}
-			else if (c == '\r') {
-				cstart = i + 1;
-				G.telstate = TS_CR;
-			}
-			/* No IACs were seen so far, no need to copy
-			 * bytes within G.buf: */
-			continue;
+	i = 0;
+	//bb_error_msg("[%u,'%.*s']", G.telstate, len, G.buf);
+	if (G.telstate == TS_NORMAL) { /* most typical state */
+		while (i < len) {
+			c = G.buf[i];
+			i++;
+			if (c == IAC) /* unlikely */
+				goto got_IAC;
+			if (c != '\r') /* likely */
+				continue;
+			G.telstate = TS_CR;
+			cstart = i;
+			goto got_special;
 		}
+		full_write(STDOUT_FILENO, G.buf, len);
+		return;
+ got_IAC:
+		G.telstate = TS_IAC;
+		cstart = i - 1;
+ got_special: ;
+	}
+
+	for (; i < len; i++) {
+		c = G.buf[i];
 
 		switch (G.telstate) {
 		case TS_CR:
@@ -278,20 +289,19 @@ static void handle_net_input(int len)
 			/* Similar to NORMAL, but in TS_COPY we need to copy bytes */
 			if (c == IAC)
 				G.telstate = TS_IAC;
-			else
+			else {
 				G.buf[cstart++] = c;
-			if (c == '\r')
-				G.telstate = TS_CR;
+				if (c == '\r')
+					G.telstate = TS_CR;
+			}
 			break;
 
 		case TS_IAC: /* Prev char was IAC */
-			if (c == IAC) { /* IAC IAC -> one IAC */
+			switch (c) {
+			case IAC: /* IAC IAC -> one IAC */
 				G.buf[cstart++] = c;
 				G.telstate = TS_COPY;
 				break;
-			}
-			/* else */
-			switch (c) {
 			case SB:
 				G.telstate = TS_SUB1;
 				break;
@@ -320,103 +330,83 @@ static void handle_net_input(int len)
 		}
 	}
 
-	if (G.telstate != TS_NORMAL) {
-		/* We had some IACs, or CR */
-		if (G.iaclen)
-			iac_flush();
-		if (G.telstate == TS_COPY) /* we aren't in the middle of IAC */
-			G.telstate = TS_NORMAL;
-		len = cstart;
-	}
-
-	if (len)
-		full_write(STDOUT_FILENO, G.buf, len);
+	/* We had some IACs, or CR */
+	iac_flush();
+	if (G.telstate == TS_COPY) /* we aren't in the middle of IAC */
+		G.telstate = TS_NORMAL;
+	if (cstart != 0)
+		full_write(STDOUT_FILENO, G.buf, cstart);
 }
 
 static void put_iac(int c)
 {
-	G.iacbuf[G.iaclen++] = c;
-}
-
-static void put_iac2_merged(unsigned wwdd_and_c)
-{
-	if (G.iaclen + 3 > IACBUFSIZE)
+	int iaclen = G.iaclen;
+	if (iaclen >= IACBUFSIZE) {
 		iac_flush();
-
-	put_iac(IAC);
-	put_iac(wwdd_and_c >> 8);
-	put_iac(wwdd_and_c & 0xff);
+		iaclen = 0;
+	}
+	G.iacbuf[iaclen] = c; /* "... & 0xff" is implicit */
+	G.iaclen = iaclen + 1;
 }
-#define put_iac2(wwdd,c) put_iac2_merged(((wwdd)<<8) + (c))
+
+static void put_iac2_msb_lsb(unsigned x_y)
+{
+	put_iac(x_y >> 8);  /* "... & 0xff" is implicit */
+	put_iac(x_y);  /* "... & 0xff" is implicit */
+}
+#define put_iac2_x_y(x,y) put_iac2_msb_lsb(((x)<<8) + (y))
+
+static void put_iac4_msb_lsb(unsigned x_y_z_t)
+{
+	put_iac2_msb_lsb(x_y_z_t >> 16);
+	put_iac2_msb_lsb(x_y_z_t);  /* "... & 0xffff" is implicit */
+}
+#define put_iac4_x_y_z_t(x,y,z,t) put_iac4_msb_lsb(((x)<<24) + ((y)<<16) + ((z)<<8) + (t))
+
+static void put_iac3_IAC_x_y_merged(unsigned wwdd_and_c)
+{
+	put_iac(IAC);
+	put_iac2_msb_lsb(wwdd_and_c);
+}
+#define put_iac3_IAC_x_y(wwdd,c) put_iac3_IAC_x_y_merged(((wwdd)<<8) + (c))
 
 #if ENABLE_FEATURE_TELNET_TTYPE
 static void put_iac_subopt(byte c, char *str)
 {
-	int len = strlen(str) + 6;   // ( 2 + 1 + 1 + strlen + 2 )
-
-	if (G.iaclen + len > IACBUFSIZE)
-		iac_flush();
-
-	put_iac(IAC);
-	put_iac(SB);
-	put_iac(c);
-	put_iac(0);
+	put_iac4_x_y_z_t(IAC, SB, c, 0);
 
 	while (*str)
 		put_iac(*str++);
 
-	put_iac(IAC);
-	put_iac(SE);
+	put_iac2_x_y(IAC, SE);
 }
 #endif
 
 #if ENABLE_FEATURE_TELNET_AUTOLOGIN
 static void put_iac_subopt_autologin(void)
 {
-	int len = strlen(G.autologin) + 6;	// (2 + 1 + 1 + strlen + 2)
-	const char *p = "USER";
+	const char *p;
 
-	if (G.iaclen + len > IACBUFSIZE)
-		iac_flush();
-
-	put_iac(IAC);
-	put_iac(SB);
-	put_iac(TELOPT_NEW_ENVIRON);
-	put_iac(TELQUAL_IS);
-	put_iac(NEW_ENV_VAR);
-
-	while (*p)
-		put_iac(*p++);
-
-	put_iac(NEW_ENV_VALUE);
+	put_iac4_x_y_z_t(IAC, SB, TELOPT_NEW_ENVIRON, TELQUAL_IS);
+	put_iac4_x_y_z_t(NEW_ENV_VAR, 'U', 'S', 'E'); /* "USER" */
+	put_iac2_x_y('R', NEW_ENV_VALUE);
 
 	p = G.autologin;
 	while (*p)
 		put_iac(*p++);
 
-	put_iac(IAC);
-	put_iac(SE);
+	put_iac2_x_y(IAC, SE);
 }
 #endif
 
 #if ENABLE_FEATURE_TELNET_WIDTH
 static void put_iac_naws(byte c, int x, int y)
 {
-	if (G.iaclen + 9 > IACBUFSIZE)
-		iac_flush();
+	put_iac3_IAC_x_y(SB, c);
 
-	put_iac(IAC);
-	put_iac(SB);
-	put_iac(c);
+	put_iac4_msb_lsb((x << 16) + y);
 
-	/* "... & 0xff" implicitly done below */
-	put_iac(x >> 8);
-	put_iac(x);
-	put_iac(y >> 8);
-	put_iac(y);
-
-	put_iac(IAC);
-	put_iac(SE);
+	put_iac2_x_y(IAC, SE);
 }
 #endif
 
@@ -445,8 +435,8 @@ static void will_charmode(void)
 	G.telflags |= (UF_ECHO | UF_SGA);
 	setConMode();
 
-	put_iac2(DO, TELOPT_ECHO);
-	put_iac2(DO, TELOPT_SGA);
+	put_iac3_IAC_x_y(DO, TELOPT_ECHO);
+	put_iac3_IAC_x_y(DO, TELOPT_SGA);
 	iac_flush();
 }
 
@@ -456,24 +446,24 @@ static void do_linemode(void)
 	G.telflags &= ~(UF_ECHO | UF_SGA);
 	setConMode();
 
-	put_iac2(DONT, TELOPT_ECHO);
-	put_iac2(DONT, TELOPT_SGA);
+	put_iac3_IAC_x_y(DONT, TELOPT_ECHO);
+	put_iac3_IAC_x_y(DONT, TELOPT_SGA);
 	iac_flush();
 }
 
 static void to_notsup(char c)
 {
 	if (G.telwish == WILL)
-		put_iac2(DONT, c);
+		put_iac3_IAC_x_y(DONT, c);
 	else if (G.telwish == DO)
-		put_iac2(WONT, c);
+		put_iac3_IAC_x_y(WONT, c);
 }
 
 static void to_echo(void)
 {
 	/* if server requests ECHO, don't agree */
 	if (G.telwish == DO) {
-		put_iac2(WONT, TELOPT_ECHO);
+		put_iac3_IAC_x_y(WONT, TELOPT_ECHO);
 		return;
 	}
 	if (G.telwish == DONT)
@@ -489,9 +479,9 @@ static void to_echo(void)
 		G.telflags ^= UF_ECHO;
 
 	if (G.telflags & UF_ECHO)
-		put_iac2(DO, TELOPT_ECHO);
+		put_iac3_IAC_x_y(DO, TELOPT_ECHO);
 	else
-		put_iac2(DONT, TELOPT_ECHO);
+		put_iac3_IAC_x_y(DONT, TELOPT_ECHO);
 
 	setConMode();
 	full_write1_str("\r\n");  /* sudden modec */
@@ -509,9 +499,9 @@ static void to_sga(void)
 
 	G.telflags ^= UF_SGA; /* toggle */
 	if (G.telflags & UF_SGA)
-		put_iac2(DO, TELOPT_SGA);
+		put_iac3_IAC_x_y(DO, TELOPT_SGA);
 	else
-		put_iac2(DONT, TELOPT_SGA);
+		put_iac3_IAC_x_y(DONT, TELOPT_SGA);
 }
 
 #if ENABLE_FEATURE_TELNET_TTYPE
@@ -519,9 +509,9 @@ static void to_ttype(void)
 {
 	/* Tell server we will (or won't) do TTYPE */
 	if (G.ttype)
-		put_iac2(WILL, TELOPT_TTYPE);
+		put_iac3_IAC_x_y(WILL, TELOPT_TTYPE);
 	else
-		put_iac2(WONT, TELOPT_TTYPE);
+		put_iac3_IAC_x_y(WONT, TELOPT_TTYPE);
 }
 #endif
 
@@ -530,9 +520,9 @@ static void to_new_environ(void)
 {
 	/* Tell server we will (or will not) do AUTOLOGIN */
 	if (G.autologin)
-		put_iac2(WILL, TELOPT_NEW_ENVIRON);
+		put_iac3_IAC_x_y(WILL, TELOPT_NEW_ENVIRON);
 	else
-		put_iac2(WONT, TELOPT_NEW_ENVIRON);
+		put_iac3_IAC_x_y(WONT, TELOPT_NEW_ENVIRON);
 }
 #endif
 
@@ -540,7 +530,7 @@ static void to_new_environ(void)
 static void to_naws(void)
 {
 	/* Tell server we will do NAWS */
-	put_iac2(WILL, TELOPT_NAWS);
+	put_iac3_IAC_x_y(WILL, TELOPT_NAWS);
 }
 #endif
 
@@ -649,6 +639,7 @@ int telnet_main(int argc UNUSED_PARAM, char **argv)
 		bb_show_usage();
 
 	xmove_fd(create_and_connect_stream_or_die(host, port), netfd);
+	printf("Connected to %s\n", host);
 
 	setsockopt_keepalive(netfd);
 
