@@ -2,15 +2,13 @@
 /*
  * DHCPv6 client.
  *
- * WARNING: THIS CODE IS INCOMPLETE.
- *
  * Copyright (C) 2011-2017 Denys Vlasenko.
  *
  * Licensed under GPLv2, see file LICENSE in this source tree.
  */
 //config:config UDHCPC6
 //config:	bool "udhcpc6 (21 kb)"
-//config:	default n  # not yet ready
+//config:	default y
 //config:	depends on FEATURE_IPV6
 //config:	help
 //config:	udhcpc6 is a DHCPv6 client
@@ -65,7 +63,7 @@
 #include <netpacket/packet.h>
 #include <linux/filter.h>
 
-/* "struct client_config_t client_config" is in bb_common_bufsiz1 */
+/* "struct client_data_t client_data" is in bb_common_bufsiz1 */
 
 static const struct dhcp_optflag d6_optflags[] = {
 #if ENABLE_FEATURE_UDHCPC6_RFC3646
@@ -206,7 +204,6 @@ static void *d6_copy_option(uint8_t *option, uint8_t *option_end, unsigned code)
 	return xmemdup(opt, opt[3] + 4);
 }
 
-
 /*** Script execution code ***/
 
 static char** new_env(void)
@@ -430,7 +427,7 @@ static char **fill_envp(const uint8_t *option, const uint8_t *option_end)
 	client6_data.env_ptr = NULL;
 	client6_data.env_idx = 0;
 
-	*new_env() = xasprintf("interface=%s", client_config.interface);
+	*new_env() = xasprintf("interface=%s", client_data.interface);
 
 	if (option)
 		option_to_env(option, option_end);
@@ -452,8 +449,8 @@ static void d6_run_script(const uint8_t *option, const uint8_t *option_end,
 	envp = fill_envp(option, option_end);
 
 	/* call script */
-	log1("executing %s %s", client_config.script, name);
-	argv[0] = (char*) client_config.script;
+	log1("executing %s %s", client_data.script, name);
+	argv[0] = (char*) client_data.script;
 	argv[1] = (char*) name;
 	argv[2] = NULL;
 	spawn_and_wait(argv);
@@ -489,7 +486,7 @@ static uint8_t *init_d6_packet(struct d6_packet *packet, char type, uint32_t xid
 	packet->d6_xid32 = xid;
 	packet->d6_msg_type = type;
 
-	clientid = (void*)client_config.clientid;
+	clientid = (void*)client_data.clientid;
 	return mempcpy(packet->d6_options, clientid, clientid->len + 2+2);
 }
 
@@ -502,7 +499,7 @@ static uint8_t *add_d6_client_options(uint8_t *ptr)
 
 	ptr += 4;
 	for (option = 1; option < 256; option++) {
-		if (client_config.opt_mask[option >> 3] & (1 << (option & 7))) {
+		if (client_data.opt_mask[option >> 3] & (1 << (option & 7))) {
 			ptr[0] = (option >> 8);
 			ptr[1] = option;
 			ptr += 2;
@@ -521,7 +518,7 @@ static uint8_t *add_d6_client_options(uint8_t *ptr)
 	ptr = mempcpy(ptr, &opt_fqdn_req, sizeof(opt_fqdn_req));
 #endif
 	/* Add -x options if any */
-	curr = client_config.options;
+	curr = client_data.options;
 	while (curr) {
 		len = (curr->data[D6_OPT_LEN] << 8) | curr->data[D6_OPT_LEN + 1];
 		ptr = mempcpy(ptr, curr->data, D6_OPT_DATA + len);
@@ -531,8 +528,9 @@ static uint8_t *add_d6_client_options(uint8_t *ptr)
 	return ptr;
 }
 
-static int d6_mcast_from_client_config_ifindex(struct d6_packet *packet, uint8_t *end)
+static int d6_mcast_from_client_data_ifindex(struct d6_packet *packet, uint8_t *end)
 {
+	/* FF02::1:2 is "All_DHCP_Relay_Agents_and_Servers" address */
 	static const uint8_t FF02__1_2[16] = {
 		0xFF, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x02,
@@ -542,7 +540,7 @@ static int d6_mcast_from_client_config_ifindex(struct d6_packet *packet, uint8_t
 		packet, (end - (uint8_t*) packet),
 		/*src*/ &client6_data.ll_ip6, CLIENT_PORT6,
 		/*dst*/ (struct in6_addr*)FF02__1_2, SERVER_PORT6, MAC_BCAST_ADDR,
-		client_config.ifindex
+		client_data.ifindex
 	);
 }
 
@@ -672,8 +670,8 @@ static NOINLINE int send_d6_discover(uint32_t xid, struct in6_addr *requested_ip
 	 */
 	opt_ptr = add_d6_client_options(opt_ptr);
 
-	bb_error_msg("sending %s", "discover");
-	return d6_mcast_from_client_config_ifindex(&packet, opt_ptr);
+	bb_info_msg("sending %s", "discover");
+	return d6_mcast_from_client_data_ifindex(&packet, opt_ptr);
 }
 
 /* Multicast a DHCPv6 request message
@@ -729,8 +727,8 @@ static NOINLINE int send_d6_select(uint32_t xid)
 	 */
 	opt_ptr = add_d6_client_options(opt_ptr);
 
-	bb_error_msg("sending %s", "select");
-	return d6_mcast_from_client_config_ifindex(&packet, opt_ptr);
+	bb_info_msg("sending %s", "select");
+	return d6_mcast_from_client_data_ifindex(&packet, opt_ptr);
 }
 
 /* Unicast or broadcast a DHCP renew message
@@ -802,15 +800,15 @@ static NOINLINE int send_d6_renew(uint32_t xid, struct in6_addr *server_ipv6, st
 	 */
 	opt_ptr = add_d6_client_options(opt_ptr);
 
-	bb_error_msg("sending %s", "renew");
+	bb_info_msg("sending %s", "renew");
 	if (server_ipv6)
 		return d6_send_kernel_packet(
 			&packet, (opt_ptr - (uint8_t*) &packet),
 			our_cur_ipv6, CLIENT_PORT6,
 			server_ipv6, SERVER_PORT6,
-			client_config.ifindex
+			client_data.ifindex
 		);
-	return d6_mcast_from_client_config_ifindex(&packet, opt_ptr);
+	return d6_mcast_from_client_data_ifindex(&packet, opt_ptr);
 }
 
 /* Unicast a DHCP release message */
@@ -832,12 +830,12 @@ int send_d6_release(struct in6_addr *server_ipv6, struct in6_addr *our_cur_ipv6)
 	if (client6_data.ia_pd)
 		opt_ptr = mempcpy(opt_ptr, client6_data.ia_pd, client6_data.ia_pd->len + 2+2);
 
-	bb_error_msg("sending %s", "release");
+	bb_info_msg("sending %s", "release");
 	return d6_send_kernel_packet(
 		&packet, (opt_ptr - (uint8_t*) &packet),
 		our_cur_ipv6, CLIENT_PORT6,
 		server_ipv6, SERVER_PORT6,
-		client_config.ifindex
+		client_data.ifindex
 	);
 }
 
@@ -903,16 +901,14 @@ static NOINLINE int d6_recv_raw_packet(struct in6_addr *peer_ipv6, struct d6_pac
 	return bytes;
 }
 
-
 /*** Main ***/
 
-static int sockfd = -1;
-
+/* Values for client_data.listen_mode */
 #define LISTEN_NONE   0
 #define LISTEN_KERNEL 1
 #define LISTEN_RAW    2
-static smallint listen_mode;
 
+/* Values for client_data.state */
 /* initial state: (re)start DHCP negotiation */
 #define INIT_SELECTING  0
 /* discover was sent, DHCPOFFER reply received */
@@ -927,7 +923,6 @@ static smallint listen_mode;
 #define RENEW_REQUESTED 5
 /* release, possibly manually requested (SIGUSR2) */
 #define RELEASED        6
-static smallint state;
 
 static int d6_raw_socket(int ifindex)
 {
@@ -986,7 +981,7 @@ static int d6_raw_socket(int ifindex)
 	log2("opening raw socket on ifindex %d", ifindex);
 
 	fd = xsocket(PF_PACKET, SOCK_DGRAM, htons(ETH_P_IPV6));
-	log2("got raw socket fd %d", fd);
+	log3("got raw socket fd %d", fd);
 
 	memset(&sock, 0, sizeof(sock)); /* let's be deterministic */
 	sock.sll_family = AF_PACKET;
@@ -1021,35 +1016,35 @@ static void change_listen_mode(int new_mode)
 			: "none"
 	);
 
-	listen_mode = new_mode;
-	if (sockfd >= 0) {
-		close(sockfd);
-		sockfd = -1;
+	client_data.listen_mode = new_mode;
+	if (client_data.sockfd >= 0) {
+		close(client_data.sockfd);
+		client_data.sockfd = -1;
 	}
 	if (new_mode == LISTEN_KERNEL)
-		sockfd = udhcp_listen_socket(/*INADDR_ANY,*/ CLIENT_PORT6, client_config.interface);
+		client_data.sockfd = udhcp_listen_socket(/*INADDR_ANY,*/ CLIENT_PORT6, client_data.interface);
 	else if (new_mode != LISTEN_NONE)
-		sockfd = d6_raw_socket(client_config.ifindex);
-	/* else LISTEN_NONE: sockfd stays closed */
+		client_data.sockfd = d6_raw_socket(client_data.ifindex);
+	/* else LISTEN_NONE: client_data.sockfd stays closed */
 }
 
 /* Called only on SIGUSR1 */
 static void perform_renew(void)
 {
-	bb_error_msg("performing DHCP renew");
-	switch (state) {
+	bb_info_msg("performing DHCP renew");
+	switch (client_data.state) {
 	case BOUND:
 		change_listen_mode(LISTEN_KERNEL);
 	case RENEWING:
 	case REBINDING:
-		state = RENEW_REQUESTED;
+		client_data.state = RENEW_REQUESTED;
 		break;
 	case RENEW_REQUESTED: /* impatient are we? fine, square 1 */
 		d6_run_script_no_option("deconfig");
 	case REQUESTING:
 	case RELEASED:
 		change_listen_mode(LISTEN_RAW);
-		state = INIT_SELECTING;
+		client_data.state = INIT_SELECTING;
 		break;
 	case INIT_SELECTING:
 		break;
@@ -1059,15 +1054,15 @@ static void perform_renew(void)
 static void perform_d6_release(struct in6_addr *server_ipv6, struct in6_addr *our_cur_ipv6)
 {
 	/* send release packet */
-	if (state == BOUND
-	 || state == RENEWING
-	 || state == REBINDING
-	 || state == RENEW_REQUESTED
+	if (client_data.state == BOUND
+	 || client_data.state == RENEWING
+	 || client_data.state == REBINDING
+	 || client_data.state == RENEW_REQUESTED
 	) {
-		bb_error_msg("unicasting a release");
+		bb_info_msg("unicasting a release");
 		send_d6_release(server_ipv6, our_cur_ipv6); /* unicast */
 	}
-	bb_error_msg("entering released state");
+	bb_info_msg("entering released state");
 /*
  * We can be here on: SIGUSR2,
  * or on exit (SIGTERM) and -R "release on quit" is specified.
@@ -1076,7 +1071,7 @@ static void perform_d6_release(struct in6_addr *server_ipv6, struct in6_addr *ou
  */
 	d6_run_script_no_option("deconfig");
 	change_listen_mode(LISTEN_NONE);
-	state = RELEASED;
+	client_data.state = RELEASED;
 }
 
 ///static uint8_t* alloc_dhcp_option(int code, const char *str, int extra)
@@ -1096,7 +1091,7 @@ static void client_background(void)
 	bb_daemonize(0);
 	logmode &= ~LOGMODE_STDIO;
 	/* rewrite pidfile, as our pid is different now */
-	write_pidfile(client_config.pidfile);
+	write_pidfile(client_data.pidfile);
 }
 #endif
 
@@ -1147,7 +1142,6 @@ static void client_background(void)
 //usage:     "\n	USR1	Renew lease"
 //usage:     "\n	USR2	Release lease"
 
-
 int udhcpc6_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 {
@@ -1176,8 +1170,13 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 	/* Default options */
 	IF_FEATURE_UDHCP_PORT(SERVER_PORT6 = 547;)
 	IF_FEATURE_UDHCP_PORT(CLIENT_PORT6 = 546;)
-	client_config.interface = "eth0";
-	client_config.script = CONFIG_UDHCPC_DEFAULT_SCRIPT;
+	client_data.interface = "eth0";
+	client_data.script = CONFIG_UDHCPC_DEFAULT_SCRIPT;
+	client_data.sockfd = -1;
+
+	/* Make sure fd 0,1,2 are open */
+	/* Set up the signal pipe on fds 3,4 - must be before openlog() */
+	udhcp_sp_setup();
 
 	/* Parse command line */
 	opt = getopt32long(argv, "^"
@@ -1189,8 +1188,8 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 		"v"
 		"\0" IF_UDHCP_VERBOSE("vv") /* -v is a counter */
 		, udhcpc6_longopts
-		, &client_config.interface, &client_config.pidfile, &str_r /* i,p */
-		, &client_config.script /* s */
+		, &client_data.interface, &client_data.pidfile, &str_r /* i,p */
+		, &client_data.script /* s */
 		, &discover_timeout, &discover_retries, &tryagain_timeout /* T,t,A */
 		, &list_O
 		, &list_x
@@ -1221,29 +1220,29 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 			n = udhcp_option_idx(optstr, d6_option_strings);
 			n = d6_optflags[n].code;
 		}
-		client_config.opt_mask[n >> 3] |= 1 << (n & 7);
+		client_data.opt_mask[n >> 3] |= 1 << (n & 7);
 	}
 	if (!(opt & OPT_o)) {
 		unsigned i, n;
 		for (i = 0; (n = d6_optflags[i].code) != 0; i++) {
 			if (d6_optflags[i].flags & OPTION_REQ) {
-				client_config.opt_mask[n >> 3] |= 1 << (n & 7);
+				client_data.opt_mask[n >> 3] |= 1 << (n & 7);
 			}
 		}
 	}
 	while (list_x) {
 		char *optstr = xstrdup(llist_pop(&list_x));
-		udhcp_str2optset(optstr, &client_config.options,
+		udhcp_str2optset(optstr, &client_data.options,
 				d6_optflags, d6_option_strings,
 				/*dhcpv6:*/ 1
 		);
 		free(optstr);
 	}
 
-	if (d6_read_interface(client_config.interface,
-			&client_config.ifindex,
+	if (d6_read_interface(client_data.interface,
+			&client_data.ifindex,
 			&client6_data.ll_ip6,
-			client_config.client_mac)
+			client_data.client_mac)
 	) {
 		return 1;
 	}
@@ -1257,8 +1256,8 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 		clientid->data[1] = 3; /* DUID-LL */
 		clientid->data[3] = 1; /* ethernet */
 		clientid_mac_ptr = clientid->data + 2+2;
-		memcpy(clientid_mac_ptr, client_config.client_mac, 6);
-		client_config.clientid = (void*)clientid;
+		memcpy(clientid_mac_ptr, client_data.client_mac, 6);
+		client_data.clientid = (void*)clientid;
 	}
 
 #if !BB_MMU
@@ -1273,16 +1272,12 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 		logmode |= LOGMODE_SYSLOG;
 	}
 
-	/* Make sure fd 0,1,2 are open */
-	bb_sanitize_stdio();
 	/* Create pidfile */
-	write_pidfile(client_config.pidfile);
+	write_pidfile(client_data.pidfile);
 	/* Goes to stdout (unless NOMMU) and possibly syslog */
-	bb_error_msg("started, v"BB_VER);
-	/* Set up the signal pipe */
-	udhcp_sp_setup();
+	bb_info_msg("started, v"BB_VER);
 
-	state = INIT_SELECTING;
+	client_data.state = INIT_SELECTING;
 	d6_run_script_no_option("deconfig");
 	change_listen_mode(LISTEN_RAW);
 	packet_num = 0;
@@ -1301,16 +1296,16 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 		/* silence "uninitialized!" warning */
 		unsigned timestamp_before_wait = timestamp_before_wait;
 
-		//bb_error_msg("sockfd:%d, listen_mode:%d", sockfd, listen_mode);
+		//bb_error_msg("sockfd:%d, listen_mode:%d", client_data.sockfd, client_data.listen_mode);
 
 		/* Was opening raw or udp socket here
-		 * if (listen_mode != LISTEN_NONE && sockfd < 0),
+		 * if (client_data.listen_mode != LISTEN_NONE && client_data.sockfd < 0),
 		 * but on fast network renew responses return faster
 		 * than we open sockets. Thus this code is moved
 		 * to change_listen_mode(). Thus we open listen socket
 		 * BEFORE we send renew request (see "case BOUND:"). */
 
-		udhcp_sp_fd_set(pfds, sockfd);
+		udhcp_sp_fd_set(pfds, client_data.sockfd);
 
 		tv = timeout - already_waited_sec;
 		retval = 0;
@@ -1339,20 +1334,20 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 			 * or if the status of the bridge changed).
 			 * Refresh ifindex and client_mac:
 			 */
-			if (d6_read_interface(client_config.interface,
-					&client_config.ifindex,
+			if (d6_read_interface(client_data.interface,
+					&client_data.ifindex,
 					&client6_data.ll_ip6,
-					client_config.client_mac)
+					client_data.client_mac)
 			) {
 				goto ret0; /* iface is gone? */
 			}
 
-			memcpy(clientid_mac_ptr, client_config.client_mac, 6);
+			memcpy(clientid_mac_ptr, client_data.client_mac, 6);
 
 			/* We will restart the wait in any case */
 			already_waited_sec = 0;
 
-			switch (state) {
+			switch (client_data.state) {
 			case INIT_SELECTING:
 				if (!discover_retries || packet_num < discover_retries) {
 					if (packet_num == 0)
@@ -1367,14 +1362,20 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 				d6_run_script_no_option("leasefail");
 #if BB_MMU /* -b is not supported on NOMMU */
 				if (opt & OPT_b) { /* background if no lease */
-					bb_error_msg("no lease, forking to background");
+					bb_info_msg("no lease, forking to background");
 					client_background();
 					/* do not background again! */
-					opt = ((opt & ~OPT_b) | OPT_f);
+					opt = ((opt & ~(OPT_b|OPT_n)) | OPT_f);
+					/* ^^^ also disables -n (-b takes priority over -n):
+					 * ifup's default udhcpc options are -R -n,
+					 * and users want to be able to add -b
+					 * (in a config file) to make it background
+					 * _and not exit_.
+					 */
 				} else
 #endif
 				if (opt & OPT_n) { /* abort if no lease */
-					bb_error_msg("no lease, failing");
+					bb_info_msg("no lease, failing");
 					retval = 1;
 					goto ret;
 				}
@@ -1395,12 +1396,12 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 				 * were seen in the wild. Treat them similarly
 				 * to "no response to discover" case */
 				change_listen_mode(LISTEN_RAW);
-				state = INIT_SELECTING;
+				client_data.state = INIT_SELECTING;
 				goto leasefail;
 			case BOUND:
 				/* 1/2 lease passed, enter renewing state */
-				state = RENEWING;
-				client_config.first_secs = 0; /* make secs field count from 0 */
+				client_data.state = RENEWING;
+				client_data.first_secs = 0; /* make secs field count from 0 */
 				change_listen_mode(LISTEN_KERNEL);
 				log1("entering renew state");
 				/* fall right through */
@@ -1423,7 +1424,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 				}
 				/* Timed out, enter rebinding state */
 				log1("entering rebinding state");
-				state = REBINDING;
+				client_data.state = REBINDING;
 				/* fall right through */
 			case REBINDING:
 				/* Switch to bcast receive */
@@ -1437,10 +1438,10 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 					continue;
 				}
 				/* Timed out, enter init state */
-				bb_error_msg("lease lost, entering init state");
+				bb_info_msg("lease lost, entering init state");
 				d6_run_script_no_option("deconfig");
-				state = INIT_SELECTING;
-				client_config.first_secs = 0; /* make secs field count from 0 */
+				client_data.state = INIT_SELECTING;
+				client_data.first_secs = 0; /* make secs field count from 0 */
 				/*timeout = 0; - already is */
 				packet_num = 0;
 				continue;
@@ -1456,10 +1457,10 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 		/* Is it a signal? */
 		switch (udhcp_sp_read()) {
 		case SIGUSR1:
-			client_config.first_secs = 0; /* make secs field count from 0 */
+			client_data.first_secs = 0; /* make secs field count from 0 */
 			already_waited_sec = 0;
 			perform_renew();
-			if (state == RENEW_REQUESTED) {
+			if (client_data.state == RENEW_REQUESTED) {
 				/* We might be either on the same network
 				 * (in which case renew might work),
 				 * or we might be on a completely different one
@@ -1482,7 +1483,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 			timeout = INT_MAX;
 			continue;
 		case SIGTERM:
-			bb_error_msg("received %s", "SIGTERM");
+			bb_info_msg("received %s", "SIGTERM");
 			goto ret0;
 		}
 
@@ -1494,15 +1495,15 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 			int len;
 
 			/* A packet is ready, read it */
-			if (listen_mode == LISTEN_KERNEL)
-				len = d6_recv_kernel_packet(&srv6_buf, &packet, sockfd);
+			if (client_data.listen_mode == LISTEN_KERNEL)
+				len = d6_recv_kernel_packet(&srv6_buf, &packet, client_data.sockfd);
 			else
-				len = d6_recv_raw_packet(&srv6_buf, &packet, sockfd);
+				len = d6_recv_raw_packet(&srv6_buf, &packet, client_data.sockfd);
 			if (len == -1) {
 				/* Error is severe, reopen socket */
 				bb_error_msg("read error: "STRERROR_FMT", reopening socket" STRERROR_ERRNO);
 				sleep(discover_timeout); /* 3 seconds by default */
-				change_listen_mode(listen_mode); /* just close and reopen */
+				change_listen_mode(client_data.listen_mode); /* just close and reopen */
 			}
 			/* If this packet will turn out to be unrelated/bogus,
 			 * we will go back and wait for next one.
@@ -1519,7 +1520,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 			continue;
 		}
 
-		switch (state) {
+		switch (client_data.state) {
 		case INIT_SELECTING:
 			if (packet.d6_msg_type == D6_MSG_ADVERTISE)
 				goto type_is_ok;
@@ -1542,15 +1543,15 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 				option = d6_find_option(packet.d6_options, packet_end, D6_OPT_STATUS_CODE);
 				if (option && (option->data[0] | option->data[1]) != 0) {
 					/* return to init state */
-					bb_error_msg("received DHCP NAK (%u)", option->data[4]);
+					bb_info_msg("received DHCP NAK (%u)", option->data[4]);
 					d6_run_script(packet.d6_options,
 							packet_end, "nak");
-					if (state != REQUESTING)
+					if (client_data.state != REQUESTING)
 						d6_run_script_no_option("deconfig");
 					change_listen_mode(LISTEN_RAW);
 					sleep(3); /* avoid excessive network traffic */
-					state = INIT_SELECTING;
-					client_config.first_secs = 0; /* make secs field count from 0 */
+					client_data.state = INIT_SELECTING;
+					client_data.first_secs = 0; /* make secs field count from 0 */
 					requested_ipv6 = NULL;
 					timeout = 0;
 					packet_num = 0;
@@ -1559,7 +1560,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 				}
 				option = d6_copy_option(packet.d6_options, packet_end, D6_OPT_SERVERID);
 				if (!option) {
-					bb_error_msg("no server ID, ignoring packet");
+					bb_info_msg("no server ID, ignoring packet");
 					continue;
 					/* still selecting - this server looks bad */
 				}
@@ -1570,7 +1571,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 				client6_data.server_id = option;
 				if (packet.d6_msg_type == D6_MSG_ADVERTISE) {
 					/* enter requesting state */
-					state = REQUESTING;
+					client_data.state = REQUESTING;
 					timeout = 0;
 					packet_num = 0;
 					already_waited_sec = 0;
@@ -1668,11 +1669,11 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 					free(client6_data.ia_na);
 					client6_data.ia_na = d6_copy_option(packet.d6_options, packet_end, D6_OPT_IA_NA);
 					if (!client6_data.ia_na) {
-						bb_error_msg("no %s option, ignoring packet", "IA_NA");
+						bb_info_msg("no %s option, ignoring packet", "IA_NA");
 						continue;
 					}
 					if (client6_data.ia_na->len < (4 + 4 + 4) + (2 + 2 + 16 + 4 + 4)) {
-						bb_error_msg("%s option is too short:%d bytes",
+						bb_info_msg("%s option is too short:%d bytes",
 							"IA_NA", client6_data.ia_na->len);
 						continue;
 					}
@@ -1681,11 +1682,11 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 							D6_OPT_IAADDR
 					);
 					if (!iaaddr) {
-						bb_error_msg("no %s option, ignoring packet", "IAADDR");
+						bb_info_msg("no %s option, ignoring packet", "IAADDR");
 						continue;
 					}
 					if (iaaddr->len < (16 + 4 + 4)) {
-						bb_error_msg("%s option is too short:%d bytes",
+						bb_info_msg("%s option is too short:%d bytes",
 							"IAADDR", iaaddr->len);
 						continue;
 					}
@@ -1696,7 +1697,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 					move_from_unaligned32(lease_seconds, iaaddr->data + 16 + 4);
 					lease_seconds = ntohl(lease_seconds);
 /// TODO: check for 0 lease time?
-					bb_error_msg("%s obtained, lease time %u",
+					bb_info_msg("%s obtained, lease time %u",
 						"IPv6", /*inet_ntoa(temp_addr),*/ (unsigned)lease_seconds);
 					address_timeout = lease_seconds;
 				}
@@ -1706,11 +1707,11 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 					free(client6_data.ia_pd);
 					client6_data.ia_pd = d6_copy_option(packet.d6_options, packet_end, D6_OPT_IA_PD);
 					if (!client6_data.ia_pd) {
-						bb_error_msg("no %s option, ignoring packet", "IA_PD");
+						bb_info_msg("no %s option, ignoring packet", "IA_PD");
 						continue;
 					}
 					if (client6_data.ia_pd->len < (4 + 4 + 4) + (2 + 2 + 4 + 4 + 1 + 16)) {
-						bb_error_msg("%s option is too short:%d bytes",
+						bb_info_msg("%s option is too short:%d bytes",
 							"IA_PD", client6_data.ia_pd->len);
 						continue;
 					}
@@ -1719,17 +1720,17 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 							D6_OPT_IAPREFIX
 					);
 					if (!iaprefix) {
-						bb_error_msg("no %s option, ignoring packet", "IAPREFIX");
+						bb_info_msg("no %s option, ignoring packet", "IAPREFIX");
 						continue;
 					}
 					if (iaprefix->len < (4 + 4 + 1 + 16)) {
-						bb_error_msg("%s option is too short:%d bytes",
+						bb_info_msg("%s option is too short:%d bytes",
 							"IAPREFIX", iaprefix->len);
 						continue;
 					}
 					move_from_unaligned32(lease_seconds, iaprefix->data + 4);
 					lease_seconds = ntohl(lease_seconds);
-					bb_error_msg("%s obtained, lease time %u",
+					bb_info_msg("%s obtained, lease time %u",
 						"prefix", /*inet_ntoa(temp_addr),*/ (unsigned)lease_seconds);
 					prefix_timeout = lease_seconds;
 				}
@@ -1745,9 +1746,9 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 					timeout = 61;
 				/* enter bound state */
 				d6_run_script(packet.d6_options, packet_end,
-					(state == REQUESTING ? "bound" : "renew"));
+					(client_data.state == REQUESTING ? "bound" : "renew"));
 
-				state = BOUND;
+				client_data.state = BOUND;
 				change_listen_mode(LISTEN_NONE);
 				if (opt & OPT_q) { /* quit after lease */
 					goto ret0;
@@ -1776,7 +1777,7 @@ int udhcpc6_main(int argc UNUSED_PARAM, char **argv)
 		perform_d6_release(&srv6_buf, requested_ipv6);
 	retval = 0;
  ret:
-	/*if (client_config.pidfile) - remove_pidfile has its own check */
-		remove_pidfile(client_config.pidfile);
+	/*if (client_data.pidfile) - remove_pidfile has its own check */
+		remove_pidfile(client_data.pidfile);
 	return retval;
 }
