@@ -28,22 +28,6 @@
 #include "libbb.h"
 #include <linux/fs.h>
 
-char BUG_wrong_field_size(void);
-#define STORE_LE(field, value) \
-do { \
-	if (sizeof(field) == 4) \
-		field = SWAP_LE32(value); \
-	else if (sizeof(field) == 2) \
-		field = SWAP_LE16(value); \
-	else if (sizeof(field) == 1) \
-		field = (value); \
-	else \
-		BUG_wrong_field_size(); \
-} while (0)
-
-#define FETCH_LE32(field) \
-	(sizeof(field) == 4 ? SWAP_LE32(field) : BUG_wrong_field_size())
-
 struct journal_params {
 	uint32_t jp_journal_1st_block;      /* where does journal start from on its device */
 	uint32_t jp_journal_dev;            /* journal device st_rdev */
